/*
 *    Copyright (c) <2002-2004> <Jean-Philippe Barrette-LaPierre>
 *    
 *    Permission is hereby granted, free of charge, to any person obtaining
 *    a copy of this software and associated documentation files 
 *    (cURLpp), to deal in the Software without restriction, 
 *    including without limitation the rights to use, copy, modify, merge,
 *    publish, distribute, sublicense, and/or sell copies of the Software,
 *    and to permit persons to whom the Software is furnished to do so, 
 *    subject to the following conditions:
 *    
 *    The above copyright notice and this permission notice shall be included
 *    in all copies or substantial portions of the Software.
 *    
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 *    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 *    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 *    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef CURLPP_CURLHANDLE_HPP
#define CURLPP_CURLHANDLE_HPP

#include "dllfct.h"
#include "cURL.hpp"

namespace cURLpp
{
  class CurlHandle
  {
  public:
    CURLPPAPI CurlHandle();

    /**
     * Simply calls curl_easy_perform on the handle,
     * but throws exceptions on errors.
     */
    void CURLPPAPI perform();

    virtual CURLPPAPI ~CurlHandle();

    /**
     * Calls curl_easy_setopt on the option.
     */
    template< typename OptionType >
    void option(cURL::CURLoption option, OptionType value);
      
    /**
     * Calls curl_easy_setopt on the option.
     */
    template< typename OptionType, cURL::CURLoption optionType >
    void option(OptionType value);

    /**
     * This function will return the cURL * handle.
     * DO NOT use this, unless you REALLY know what you
     * are doing.
     */
    cURL::CURL * CURLPPAPI getHandle();

  private:
    CURLPPAPI CurlHandle(const CurlHandle &other);
    CURLPPAPI CurlHandle& operator=(const CurlHandle &other);

    /**
     * Pass a char * to a buffer that the libcURL may store
     * human readable error messages in. This may  be  more
     * helpful  than just the return code from the library.
     * The buffer must be at least CURL_ERROR_SIZE big.
     * Note: if the library does not return an  error,  the
     * buffer may not have been touched. Do not rely on the
     * contents in those cases.
     */
    void CURLPPAPI errorBuffer(char* buffer);
    
    
  private:
    cURL::CURL *mCurl;
    
    char mErrorBuffer[CURL_ERROR_SIZE + 1];
  };
}

#include "CurlHandle.inl"

#endif
