/*
 *    Copyright (c) <2002-2004> <Jean-Philippe Barrette-LaPierre>
 *    
 *    Permission is hereby granted, free of charge, to any person obtaining
 *    a copy of this software and associated documentation files 
 *    (cURLpp), to deal in the Software without restriction, 
 *    including without limitation the rights to use, copy, modify, merge,
 *    publish, distribute, sublicense, and/or sell copies of the Software,
 *    and to permit persons to whom the Software is furnished to do so, 
 *    subject to the following conditions:
 *    
 *    The above copyright notice and this permission notice shall be included
 *    in all copies or substantial portions of the Software.
 *    
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 *    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 *    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 *    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "cURL.hpp"
#include "Option.hpp"
#include "StorageOptions.hpp"

namespace cURLpp
{
  namespace Options
  {
    /**
     * Behavior options.
     */
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_VERBOSE > Verbose;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_HEADER > Header;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_NOSIGNAL > NoSignal;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_NOPROGRESS > NoProgress;

    /**
     * Callback options. 
     */
    typedef cURLpp::OptionTrait< cURL::curl_write_callback, cURL::CURLOPT_WRITEFUNCTION > WriteFunction;
    typedef cURLpp::OptionTrait< void *, cURL::CURLOPT_WRITEDATA > WriteData;
    typedef cURLpp::OptionTrait< cURL::curl_read_callback, cURL::CURLOPT_READFUNCTION > ReadFunction;
    typedef cURLpp::OptionTrait< void *, cURL::CURLOPT_READDATA > ReadData;
    typedef cURLpp::OptionTrait< cURL::curl_progress_callback, cURL::CURLOPT_PROGRESSFUNCTION > ProgressFunction;
    typedef cURLpp::OptionTrait< void *, cURL::CURLOPT_PROGRESSDATA > ProgressData;
    typedef cURLpp::OptionTrait< cURL::curl_write_callback, cURL::CURLOPT_HEADERFUNCTION > HeaderFunction;
    typedef cURLpp::OptionTrait< void *, cURL::CURLOPT_HEADERDATA > HeaderData;
    typedef cURLpp::OptionTrait< cURL::curl_debug_callback, cURL::CURLOPT_DEBUGFUNCTION > DebugFunction;
    typedef cURLpp::OptionTrait< void *, cURL::CURLOPT_DEBUGDATA > DebugData;
#ifdef CURLOPT_SSL_CTX_FUNCTION
    typedef cURLpp::OptionTrait< cURL::curl_ssl_ctx_callback, cURL::CURLOPT_SSL_CTX_FUNCTION > SslCtxFunction;
    typedef cURLpp::OptionTrait< void *, cURL::CURLOPT_SSL_CTX_DATA > SslCtxData;
#endif
      
      
    /**
     * Error options.
     */
    typedef cURLpp::OptionTrait< char *, cURL::CURLOPT_ERRORBUFFER > ErrorBuffer;
#ifdef FILE
    typedef cURLpp::OptionTrait< FILE *, cURL::CURLOPT_STDERR > StdErr;
#endif
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_FAILONERROR > FailOnError;

    /**
     * Network options.
     */
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_URL > Url;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_PROXY > Proxy;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_PROXYPORT > ProxyPort;
    typedef cURLpp::OptionTrait< cURL::curl_proxytype, cURL::CURLOPT_PROXYTYPE > ProxyType;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_HTTPPROXYTUNNEL > HttpProxyTunnel;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_INTERFACE > Interface;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_DNS_CACHE_TIMEOUT > DnsCacheTimeout;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_DNS_USE_GLOBAL_CACHE > DnsUseGlobalCache;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_BUFFERSIZE > BufferSize;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_PORT > Port;
#ifdef CURLOPT_TCP_NODELAY
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_TCP_NODELAY > TcpNoDelay;
#endif

    /**
     * Names and passwords options.
     */
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_NETRC > Netrc;
#ifdef CURLOPT_NETRC_FILE
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_NETRC_FILE > NetrcFile;
#endif
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_USERPWD > UserPwd;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_PROXYUSERPWD > ProxyUserPwd;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_HTTPAUTH > HttpAuth;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_PROXYAUTH > ProxyAuth;

    /**
     * HTTP options.
     */
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_AUTOREFERER > AutoReferer;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_ENCODING > Encoding;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_FOLLOWLOCATION > FollowLocation;
#ifdef CURLOPT_UNRESTRICTED_AUTH
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_UNRESTRICTED_AUTH > UnrestrictedAuth;
#endif
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_MAXREDIRS > MaxRedirs;
#ifndef CURLOPT_UPLOAD
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_PUT > Put;
#else
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_UPLOAD > Put;
#endif
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_POST > Post;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_POSTFIELDS > PostFields;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_POSTFIELDSIZE > PostFieldSize;
#ifdef CURLOPT_POSTFIELDSIZE_LARGE
    typedef cURLpp::OptionTrait< cURL::curl_off_t, cURL::CURLOPT_POSTFIELDSIZE_LARGE > PostFieldSizeLarge;
#endif
    typedef cURLpp::OptionTrait< std::list< cURLpp::FormPart * >, cURL::CURLOPT_HTTPPOST > HttpPost;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_REFERER > Referer;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_USERAGENT > UserAgent;
    typedef cURLpp::OptionTrait< std::list< std::string >, cURL::CURLOPT_HTTPHEADER > HttpHeader;
    typedef cURLpp::OptionTrait< std::list< std::string >, cURL::CURLOPT_HTTP200ALIASES > Http200Aliases;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_COOKIE > Cookie;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_COOKIEFILE > CookieFile;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_COOKIEJAR > CookieJar;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_COOKIESESSION > CookieSession;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_HTTPGET > HttpGet;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_HTTP_VERSION > HttpVersion;

    /**
     * FTP options.
     */
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_FTPPORT > FtpPort;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_QUOTE > Quote;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_POSTQUOTE > PostQuote;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_PREQUOTE > PreQuote;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_FTPLISTONLY > FtpListOnly;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_FTPAPPEND > FtpAppend;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_FTP_USE_EPSV > FtpUseEpsv;
#ifdef CURLOPT_FTP_CREATE_MISSING_DIRS
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_FTP_CREATE_MISSING_DIRS > FtpCreateMissingDirs;
#endif
#ifdef CURLOPT_FTP_RESPONSE_TIMEOUT
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_FTP_RESPONSE_TIMEOUT > FtpResponseTimeout;
#endif

#ifdef CURLOPT_FTP_SSL
    typedef cURLpp::OptionTrait< cURL::curl_ftpssl, cURL::CURLOPT_FTP_SSL > FtpSsl;
#endif
#ifdef CURLOPT_FTP_AUTH
    typedef cURLpp::OptionTrait< cURL::curl_ftpauth, cURL::CURLOPT_FTP_AUTH > FtpAuth;
#endif
      
    /**
     * Protocol options.
     */
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_TRANSFERTEXT > TransferText;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_CRLF > Crlf;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_RANGE > Range;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_RESUME_FROM > ResumeFrom;
#ifdef CURLOPT_RESUME_FROM_LARGE
    typedef cURLpp::OptionTrait< cURL::curl_off_t, cURL::CURLOPT_RESUME_FROM_LARGE > ResumeFromLarge;
#endif
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_CUSTOMREQUEST > CustomRequest;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_FILETIME > FileTime;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_NOBODY > NoBody;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_INFILESIZE > InfileSize;
#ifdef CURLOPT_INFILESIZE_LARGE
    typedef cURLpp::OptionTrait< cURL::curl_off_t, cURL::CURLOPT_INFILESIZE_LARGE > InfileSizeLarge;
#endif
#ifdef CURLOPT_UPLOAD
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_UPLOAD > Upload;
#else
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_PUT > Upload;
#endif
#ifdef CURLOPT_MAXFILESIZE
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_MAXFILESIZE > MaxFileSize;
#endif
#ifdef CURLOPT_MAXFILESIZE_LARGE
    typedef cURLpp::OptionTrait< cURL::curl_off_t, cURL::CURLOPT_MAXFILESIZE_LARGE > MaxFileSizeLarge;
#endif
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_TIMECONDITION > TimeCondition;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_TIMECONDITION > TimeValue;

    /**
     * Connection options.
     */
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_TIMEOUT > Timeout;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_LOW_SPEED_LIMIT > LowSpeedLimit;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_LOW_SPEED_TIME > LowSpeedTime;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_MAXCONNECTS > MaxConnects;
    typedef cURLpp::OptionTrait< cURL::curl_closepolicy, cURL::CURLOPT_CLOSEPOLICY > ClosePolicy;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_FRESH_CONNECT > FreshConnect;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_FORBID_REUSE > ForbidReuse;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_CONNECTTIMEOUT > ConnectTimeout;
#ifdef CURLOPT_IPRESOLVE
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_IPRESOLVE > IpResolve;
#endif
      
    /**
     * SSL and security options.
     */
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_SSLCERT > SslCert;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_SSLCERTTYPE > SslCertType;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_SSLCERTPASSWD > SslCertPasswd;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_SSLKEY > SslKey;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_SSLKEYTYPE > SslKeyType;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_SSLKEYPASSWD > SslKeyPasswd;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_SSLENGINE > SslEngine;
    //SSLEngineDefault;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_SSLVERSION > SslVersion;
    typedef cURLpp::OptionTrait< bool, cURL::CURLOPT_SSL_VERIFYPEER > SslVerifyPeer;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_CAINFO > CaInfo;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_CAPATH > CaPath;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_RANDOM_FILE > RandomFile;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_EGDSOCKET > EgdSocket;
    typedef cURLpp::OptionTrait< long, cURL::CURLOPT_SSL_VERIFYHOST > SslVerifyHost;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_SSL_CIPHER_LIST > SslCipherList;
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_KRB4LEVEL > Krb4Level;

    /**
     * Others options.
     */
    //Private
    typedef cURLpp::OptionTrait< std::string, cURL::CURLOPT_KRB4LEVEL > Krb4Level;
    //Share;
    //TelnetOptions
  }
}
